@@ -4,11 +4,13 @@
 #pragma once
 
 #include "ISPCDeviceObject.h"
+#include "common/FeatureFlagsEnum.h"
 #include "common/ObjectFactory.h"
 #include "common/StructShared.h"
 // ispc shared
 #include "MaterialShared.h"
 #include "bsdfs/MicrofacetAlbedoTables.h"
+#include "texture/Texture.h"
 #include "texture/TextureParamShared.h"
 
 namespace ospray {
@@ -19,6 +21,7 @@ struct MaterialParam
 {
   T factor;
   ispc::TextureParam tex;
+  Ref<Texture> ref_tex;
   linear2f rot;
 };
 
@@ -29,19 +32,22 @@ struct OSPRAY_SDK_INTERFACE Material
     : public AddStructShared<ISPCDeviceObject, ispc::Material>,
       public ObjectFactory<Material, api::ISPCDevice &>
 {
-  Material(api::ISPCDevice &device);
+  Material(api::ISPCDevice &device, const FeatureFlagsOther ffo);
   virtual ~Material() override;
   virtual std::string toString() const override;
   virtual void commit() override;
 
+  FeatureFlags getFeatureFlags() const;
+
   // helper function to get all texture related parameters
-  ispc::TextureParam getTextureParam(const char *texture_name);
+  ispc::TextureParam getTextureParam(
+      const char *texture_name, const Ref<Texture> &ref_tex);
 
   /*! \brief helper function to get a uniform or texture material parameter */
   MaterialParam1f getMaterialParam1f(const char *name, float valIfNotFound);
   MaterialParam3f getMaterialParam3f(const char *name, vec3f valIfNotFound);
 
-  bool isEmissive() const;
+  virtual bool isEmissive() const;
 
  private:
   /*! \brief helper function to combine multiple texture transformation
@@ -64,6 +70,8 @@ struct OSPRAY_SDK_INTERFACE Material
   utility::Optional<affine2f> getTextureTransform2f(const char *_texname);
   utility::Optional<affine3f> getTextureTransform3f(const char *_texname);
 
+  FeatureFlagsOther featureFlags;
+
   // Microfacet albedo tables data, shared by all materials. New materials
   // increment the use count, and decrement it on destruction to ensure
   // the data will be released before the device
@@ -74,8 +82,16 @@ OSPTYPEFOR_SPECIALIZATION(Material *, OSP_MATERIAL);
 
 // Inlined definitions /////////////////////////////////////////////////////////
 
+inline FeatureFlags Material::getFeatureFlags() const
+{
+  FeatureFlags ff;
+  ff.other = featureFlags;
+  return ff;
+}
+
 inline bool Material::isEmissive() const
 {
-  return reduce_max(getSh()->emission) > 0.f;
+  return false;
 }
+
 } // namespace ospray
